#include <stdio.h>    /* for printf */
#include <stdlib.h>   /* for malloc */
 
typedef struct node {
	int data;
	struct node *next; /* pointer to next element in list */
} LLIST;
 
LLIST *list_add(LLIST **p, int i);
void list_remove(LLIST **p);
LLIST **list_search(LLIST **n, int i);
void list_print(LLIST *n);
void list_insert_after(LLIST *n, int i);
void list_insert_beginning(LLIST **n, int i);
void list_remove_all(LLIST **p);
void list_remove_after(LLIST **n);
 
 
LLIST *list_add(LLIST **p, int i)
{
	if (p == NULL)
		return NULL;
 
	LLIST *n = malloc(sizeof(LLIST));
	if (n == NULL)
		return NULL;
 
	n->next = *p; /* the previous element (*p) now becomes the "next" element */
	*p = n;       /* add new empty element to the front (head) of the list */
	n->data = i;
 
	return *p;
}
 
void list_remove(LLIST **p) /* remove head */
{
	if (p != NULL && *p != NULL)
	{
		LLIST *n = *p;
		*p = (*p)->next;
		free(n);
	}
}
 
LLIST **list_search(LLIST **n, int i)
{
	if (n == NULL)
		return NULL;
 
	while (*n != NULL)
	{
		if ((*n)->data == i)
		{
			return n;
		}
		n = &(*n)->next;
	}
	return NULL;
}
 
void list_print(LLIST *n)
{
	if (n == NULL)
	{
		printf("list is empty\n");
	}
	while (n != NULL)
	{
		printf("print %p %p %d\n", n, n->next, n->data);
		n = n->next;
	}
	printf("\n");
}
 
void list_insert_after(LLIST *n, int i)
{
    if (n == NULL)
       return;
    
   	LLIST *nn = malloc(sizeof(LLIST));
	if (nn == NULL)
		return;
 
	nn->data = i;
	nn->next = n->next; 
    n->next = nn;
} 

void list_insert_beginning(LLIST **n, int i)
{
    if (*n == NULL)
       return;
    
   	LLIST *nn = malloc(sizeof(LLIST));
	if (nn == NULL)
		return;
 
	nn->data = i;
	nn->next = *n; 
    *n = nn;
} 

void list_remove_all(LLIST **p)
{
     LLIST *temp = *p;
     for ( ;temp != NULL; temp = *p) {
         *p = (*p)->next;
         free(temp);
     }
     *p = NULL;
}

void list_remove_after(LLIST **n) {

	if (*n != NULL)
	{
		LLIST *temp = (*n)->next;
		(*n)->next = temp->next;
		free(temp);
	}
}
     

int main(void)
{
	LLIST *n = NULL;
	LLIST **nn;
 
	list_add(&n, 0); /* list: 0 */
	list_add(&n, 1); /* list: 1 0 */
	list_add(&n, 2); /* list: 2 1 0 */
	list_add(&n, 3); /* list: 3 2 1 0 */
	list_add(&n, 4); /* list: 4 3 2 1 0 */
	list_print(n);
	
	nn = list_search(&n, 2);
	list_insert_after(*nn,11);
	list_print(n);
	
	list_insert_beginning(&n, 77);
	list_print(n);
	
	list_remove_after(list_search(&n, 3));
	list_print(n);
	
	list_remove(&n);                 /* remove first (4) */
	list_remove(&n->next);           /* remove new second (2) */
	list_remove(list_search(&n, 1)); /* remove cell containing 1 (first) */
	list_remove(&n->next);           /* remove second to last node (0) */
	list_remove(&n);                 /* remove last (3) */
    list_print(n);
    
    list_remove_all(&n);
    list_print(n);
 
    system("pause");
	return 0;
}
